#!/usr/bin/env node

import inquirer from "inquirer";
import fs from "fs-extra";
import path from "path";
import { execa } from "execa";
import process from "process";
import { fileURLToPath } from "url";

import {
  tsconfig,
  tsDevInstallers,
  tsInstallers,
} from "./constants/typescript-constants.js";
import {
  answersCheck,
  paths as pathname,
  questions,
  scripts,
} from "./constants/global-constants.js";
import {
  jsDevInstallers,
  jsInstallers,
} from "./constants/javascript-constants.js";
import { startLoading, stopLoading } from "./animations/loading.animation.js";

const __dirname = path.dirname(fileURLToPath(import.meta.url));

inquirer
  .prompt(questions)
  .then(async (answers) => {
    const targetPath = process.cwd();
    const { isJavaScript, isModule } = answersCheck(answers);
    const paths = pathname(
      targetPath,
      answers.language,
      __dirname,
      answers.installDir,
      answersCheck(answers)
    );
    console.log("Initializing project please wait ...");
    await startLoading();
    await fs.copy(paths.templatePath, targetPath);
    await execa("npm", ["init", "-y"], { cwd: paths.basePath });

    await fs.writeFile(paths.dotenvFilePath, "");

    let packageJson = await fs.readFile(paths.packageJsonPath, "utf8");
    packageJson = JSON.parse(packageJson);
    packageJson.type = !isModule ? "commonjs" : "module";
    packageJson.scripts = scripts(answers);
    packageJson.name =
      answers.projectName === "./"
        ? path.basename(targetPath)
        : answers.projectName;
    await fs.writeFile(
      paths.packageJsonPath,
      JSON.stringify(packageJson, null, 2)
    );

    if (isJavaScript) {
      await execa("npm", jsInstallers, { cwd: paths.basePath });
      await execa("npm", jsDevInstallers, { cwd: paths.basePath });
    } else {
      await execa("npm", tsInstallers, { cwd: paths.basePath });
      await execa("npm", tsDevInstallers, { cwd: paths.basePath });
    }

    if (!isJavaScript) {
      await fs.writeFile(
        paths.tsconfigPath,
        JSON.stringify(tsconfig(isJavaScript), null, 2)
      );
    }

    await fs.copyFile(paths.sourceFilePath, paths.indexFilePath);
    await fs.copyFile(paths.eslintFileCopyPath, paths.eslintConfigPath);
    await stopLoading();
    console.log("Project created successfully!");
    console.log("to run in dev mode -> npm run dev:watch");
    console.log("to start           -> npm start");
  })
  .catch((error) => {
    console.error("Error:", error);
  });
